@@ -41,7 +41,9 @@
 
 #include <QtCore/QDateTime>
 #include <QtCore/QDataStream>
+#include <QtCore/QDir>
 #include <QtCore/QFile>
+#include <QtCore/QFileInfo>
 #include <QtCore/QVariant>
 #include <QtCore/QList>
 
@@ -248,6 +250,8 @@ bool KTNEFParser::ParserPrivate::decodeMessage()
         quint32 rows;
         QList<QVariant> recipTable;
         stream_ >> rows;
+        if (rows > (INT_MAX / sizeof(QVariant)))
+            return false;
         recipTable.reserve(rows);
         for (uint i = 0; i < rows; i++) {
             QMap<int, KTNEFProperty *> props;
@@ -444,7 +448,9 @@ bool KTNEFParser::extractFile(const QString &filename) const
 bool KTNEFParser::ParserPrivate::extractAttachmentTo(KTNEFAttach *att,
         const QString &dirname)
 {
-    QString filename = dirname + QLatin1Char('/');
+    const QString destDir(QDir(dirname).absolutePath()); // get directory path without any "." or ".."
+
+    QString filename = destDir + QLatin1Char('/');
     if (!att->fileName().isEmpty()) {
         filename += att->fileName();
     } else {
@@ -460,6 +466,15 @@ bool KTNEFParser::ParserPrivate::extractAttachmentTo(KTNEFAttach *att,
     if (!device_->seek(att->offset())) {
         return false;
     }
+
+    const QFileInfo fi(filename);
+    if (!fi.absoluteFilePath().startsWith(destDir)) {
+        qWarning() << "Attempted extract into" << fi.absoluteFilePath()
+                   << "which is outside of the extraction root folder" << destDir << "."
+                   << "Changing export of contained files to extraction root folder.";
+        filename = destDir + QLatin1Char('/') + fi.fileName();
+    }
+
     QSaveFile outfile(filename);
     if (!outfile.open(QIODevice::WriteOnly)) {
         return false;
@@ -706,6 +721,9 @@ QString readMAPIString(QDataStream &stream, bool isUnicode, bool align,
     } else {
         len = len_;
     }
+    if (len > INT_MAX)
+        return QString();
+
     quint32 fullLen = len;
     if (align) {
         ALIGN(fullLen, 4);
@@ -806,17 +824,17 @@ quint16 readMAPIValue(QDataStream &stream, MAPI_value &mapi)
             } else {
                 stream >> d;
             }
-            for (uint i = 0; i < d; i++) {
+            for (uint i = 0; i < d && !stream.atEnd(); i++) {
                 value.clear();
                 quint32 len;
                 stream >> len;
                 value = QByteArray(len, '\0');
-                if (len > 0) {
-                    int fullLen = len;
+                if (len > 0 && len <= INT_MAX) {
+                    uint fullLen = len;
                     ALIGN(fullLen, 4);
                     stream.readRawData(value.toByteArray().data(), len);
                     quint8 c;
-                    for (int i = len; i < fullLen; i++) {
+                    for (uint i = len; i < fullLen; i++) {
                         stream >> c;
                     }
                     // FIXME: Shouldn't we do something with the value???
